<?php   
	/*
	 * Functions for showing Google Picasa Webalbums within your Concrete 5 website
	 * within your Concrete 5 website with the use of Highslide http//:highslide.com
	 * @author Wim Bouter <wim@dewebmakers.nl>
	 */
	 
	Loader::block('library_file');	
	defined('C5_EXECUTE') or die(_("Access Denied."));
	
	class HighslideWebAlbumsBlockController extends BlockController {
 
		protected $btInterfaceWidth = 500;
		protected $btInterfaceHeight = 550;
		protected $btTable = 'btHighslideWebAlbum';
		
		public $thumb_attribute = 0;
		public $thumb_width = 70;
		public $thumb_width_crop  = 1;
		public $max_width = 1024;
		public $max_width_crop = 0;
		
		public $dimming = 1;
		public $dimmingopacity = 75;
		
		public $credits = 0;
		public $credits_url = 'http://www.concrete5.org';
		public $credits_title = 'Concrete 5 - Highslide Webalbum';
		public $credits_text = 'Concrete 5 - Highslide Webalbum';
		
		public $playback = "ORDER";	
		
		public $custom_js = '';
		public $custom_css = '';
		
		public $images = array();
		
		public $attributes = array(array('akHandle' => 'fvTitle', 'akName' => 'Title'),
								   array('akHandle' => 'fvDescription', 'akName' => 'Description'),
								   array('akHandle' => 'fvTags', 'akName' => 'Tags'),
								   array('akHandle' => 'fvFilename', 'akName' => 'Filename'),
								   array('akHandle' => 'fvSize', 'akName' => 'Size'),
								   array('akHandle' => 'fvDateAdded', 'akName' => 'Date Added'),
								   array('akHandle' => 'fvAuthor', 'akName' => 'Author'),
								   array('akHandle' => 'fvDownload', 'akName' => 'Download URL'));

								   
		public $alttags_attributes = array();
		public $selected_attributes = array();		
		
		public $alttags = 1;
		public $alttags_title = 1;
		
		public $type = 'CUSTOM';
		
		public $pagination = 0;
		public $pagination_number = 20;
		
		public function getBlockTypeDescription() {
			return t("Shows your images (or imageset) with the use of Highslide Gallery");
		}
		
		public function getBlockTypeName() {
			return t("Highslide Web Albums");
		}
		
		public function getJavaScriptStrings() {
			return array(
				'choose-file' => t('Choose Image/File'),
				'choose-min-2' => t('Please choose at least one image.'),
				'choose-fileset' => t('Please choose a file set.')
			);
		}
	
		function __construct($obj = null) {	
			parent::__construct($obj);
			$this->db = Loader::db();
			
			$this->set('bID', $this->bID);
			
			$this->loadAttributes();			
			$this->getAttributes();
			$this->alttags_attribs = @explode('%%', $this->alttags_attribs);
			
			if ($this->fsID == 0)
				$this->loadImages();
			else
				$this->loadFileSet();
			
			Loader::library('item_list');
			
			if (!empty($this->pagination) && intval($this->pagination_number) > 0) 
			{					
				$imageList = new ItemList();
				$imageList->setItemsPerPage(intval($this->pagination_number));
				$imageList->setItems($this->images);
				
				$this->images = $imageList->getPage();
					
				$this->set('pagination', $imageList->getPagination());
			}
				
			$this->randomizeImages();						
			
			$this->type = ($this->fsID > 0) ? 'FILESET' : 'CUSTOM';			
		}
		
		function getFileSetName(){
			$sql = "SELECT fsName FROM FileSets WHERE fsID=".intval($this->fsID);
			return $this->db->getOne($sql); 
		}
		
		function loadAttributes(){			
			Loader::model('attribute/categories/file');			
			$otypes = FileAttributeKey::getList();
			foreach ($otypes as $type)
			{
				$types[] = array('akHandle' => $type->akHandle,
								 'akName' => $type->akName);
			}			
			$this->attributes = @array_merge((array)$this->attributes, (array)$types);
		}
		
		function getAttributes(){									
			$attributes = @explode("%%", $this->alttags_attribs);
			$this->selected_attributes = $attributes;
		}
			
		function loadFileSet(){
			
			if (intval($this->fsID) < 1) {
				return false;
			}
			$f = Loader::helper('concrete/file');

			$sql = "SELECT fsf.fID, 
						   fv.fvFilename, 
						   fv.fvPrefix
				    FROM FileSetFiles AS fsf, 
						 FileVersions AS fv
				    WHERE fsf.fsID = " . $this->fsID . " 
					AND fsf.fID = fv.fID 
					AND fvIsApproved = 1
					ORDER BY fsDisplayOrder ASC";
				   
			$files = $this->db->getAll($sql); 
			
			$image = array();
			$image['groupSet'] = 0;
			$image['alttag'] = '';
			$images = array();
			$maxHeight = 0;
			foreach ($files as $file) {
				$image['fID'] = $file['fID']; 
				$image['fileName'] = $file['fvFilename'];
				$image['fullFilePath'] = $f->getFileRelativePath($file['fvPrefix'], $file['fvFilename']);
				
				$images[] = $this->prepareImage($image);
			}
			$this->images = $images;
		}
	
		function loadImages(){
			if(intval($this->bID)==0) $this->images=array();
			$sortChoices=array('ORDER'=>'position', 
							   'RANDOM-SET'=>'groupSet asc, position asc',
							   'RANDOM'=>'rand()');
			if( !array_key_exists($this->playback,$sortChoices) ) 
				$this->playback='ORDER';
			if(intval($this->bID)==0) return array();
			$sql = "SELECT * 
					FROM btHighslideWebAlbumImg 
					WHERE bID=".intval($this->bID).' 
					ORDER BY '.$sortChoices[$this->playback];		
			$files = $this->db->getAll($sql);
			foreach ($files as $file) {
				$images[] = $this->prepareImage($file);
			}
			$this->images = $images;
		}
		
		function delete(){
			$this->db->query("DELETE FROM btHighslideWebAlbumImg WHERE bID=".intval($this->bID));		
			parent::delete();
		}
		
		function duplicate($nbID) {
			parent::duplicate($nbID);
			$db = Loader::db();
			
			$this->db->query("DELETE FROM btHighslideWebAlbumImg WHERE bID=".intval($this->bID));			
			if( $this->type == 'CUSTOM' && count($this->images) ){				
				foreach($this->images as $im) {
					$db->Execute('insert into btHighslideWebAlbumImg (bID, fID, groupSet, position) values (?, ?, ?, ?)', 
						array($nbID, $im['fID'], $im['groupSet'], $im['position'])
					);		
				}
			}
		}
	
		function save($data) { 
			
			$args['playback'] = isset($data['playback']) ? trim($data['playback']) : 'ORDER';
			
			$args['thumb_attribute'] = (intval($data['thumb_attribute']) > 0)  ? intval($data['thumb_attribute']) : 0;
			
			$args['thumb_width'] = (intval($data['thumb_width']) > 0)  ? intval($data['thumb_width']) : 70;
			
			$args['thumb_width_crop'] = 0;
			if ($data['thumb_width_crop'] == 1)
				$args['thumb_width_crop'] = $data['thumb_width_crop'];	
				
			$args['max_width'] = (intval($data['max_width']) > 0) ? intval($data['max_width']) : 800;
			
			$args['max_width_crop'] = 0;
			if ($data['max_width_crop'] == 1)
				$args['max_width_crop'] = $data['max_width_crop'];
			
			$args['alttags'] = 0;					
			if ($data['alttags'] == 1) 
			{
				$args['alttags'] = $data['alttags'];
					
				$args['alttags_title'] = 0;
				if ($data['alttags_title'] == 1)
					$args['alttags_title'] = $data['alttags_title'];
						
				$args['alttags_attribs'] = @implode('%%', (array)$data['alttags_attribs']);				
			}
			
			if (intval($data['dimming']) != 1)
				$data['dimming'] = 0;
				
			$args['dimming']  	      = $data['dimming'];			
			$args['dimmingcolor']     = $data['dimmingcolor'];
			$args['dimmingopacity']   = $data['dimmingopacity'];
			
			$args['credits'] = 0;					
			if ($data['credits'] == 1) 
			{
				$args['credits'] = $data['credits'];
				$args['credits_url']  	  = $data['credits_url'];
				$args['credits_title']    = $data['credits_title'];
				$args['credits_text']  	  = $data['credits_text'];	
			}
			
			if (intval($data['pagination']) != 1)
				$data['pagination'] = 0;
			
			$args['pagination'] = $data['pagination'];
			$args['pagination_number'] = intval($data['pagination_number']);
			
			$args['custom_css'] = '';
			if (strlen(trim($data['custom_css'])) > 10)
				$args['custom_css'] = serialize(trim($data['custom_css']));
				
			$args['custom_js'] = '';
			if (strlen(trim($data['custom_js'])) > 10)
				$args['custom_js'] = serialize(trim($data['custom_js']));
			
			if( $data['type'] == 'FILESET' && $data['fsID'] > 0){
				$args['fsID'] = $data['fsID'];
	
				$files = $this->db->getAll("SELECT fv.fID 
											FROM FileSetFiles fsf, 
												 FileVersions fv 
										    WHERE fsf.fsID = " . $data['fsID'] ."
											AND fsf.fID = fv.fID 
											AND fvIsApproved = 1");
				
				//delete existing images
				$this->db->query("DELETE FROM btHighslideWebAlbumImg 
								  WHERE bID = ".intval($this->bID));
			
			} else if( $data['type'] == 'CUSTOM' && count($data['imgFIDs']) ){
				
				$args['fsID'] = 0;
	
				//delete existing images
				$this->db->query("DELETE FROM btHighslideWebAlbumImg 
								  WHERE bID=".intval($this->bID));
				
				//loop through and add the images
				$pos=0;
				foreach($data['imgFIDs'] as $imgFID){ 
					if(intval($imgFID)==0 || $data['fileNames'][$pos]=='tempFilename') continue;
					$vals = array(intval($this->bID),intval($imgFID),intval($data['groupSet'][$pos]), $pos);
					$this->db->query("INSERT INTO btHighslideWebAlbumImg (bID,fID,groupSet,position) values (?,?,?,?)",$vals);
					$pos++;
				}
			}
			
			parent::save($args);
		}				
		
		function view(){ 
			
			$this->set('images', (array)$this->images);
			$this->set('show_alttags_title', $this->alttags_title);
		}
	
		public function on_page_view() {
			
			if (!defined('HIGHSLIDE_LOADED_JS')) {
				define('HIGHSLIDE_LOADED_JS', true);
				
				$html = Loader::helper('html');
				
				$this->addFooterItem($html->javascript('highslide.js', 'highslide_web_albums'));
				$this->addFooterItem($html->javascript('highslide.mobile.js', 'highslide_web_albums'));
				
				$this->addHeaderItem($html->css('highslide.css', 'highslide_web_albums'));
				$this->addHeaderItem("<!--[if lt IE 7]>\n".$html->css('highslide-ie6.css', 'highslide_web_albums')."\n<![endif]-->\n");				
				
				if (strlen($this->dimmingopacity) == 1)
					$dimmingopacity = '0'.$this->dimmingopacity;
				
				$cjc = unserialize($this->custom_css);
				if (strlen($cjc) > 0)
				{
					$css .= "<style>\n";
					$css .= str_replace('url(graphics/', 'url('.DIR_REL.'/packages/highslide_web_albums/images/', $cjc)."\n";
					$css .= "</style>\n";
						
					$this->addHeaderItem($css);
				}
				$script .= "<script type=\"text/javascript\">\n". 
						   "// 6058e2658aaf2fe11d7bbe3242936e32\n";
				
				$cjs = unserialize($this->custom_js);
				if (strlen($cjs) > 0)
				{
					$script .= str_replace(array('highslide/graphics/', 'config1'), array(DIR_REL.'/packages/highslide_web_albums/images/', 'config_'.$this->bID), $cjs);					
					if (strpos($script, 'slideshowGroup: \'group') !== false)
						$this->set('is_gallery', true);
				}
				else
				{					
					$script .=	"hs.lang = {\n".
							   "\tloadingText : '".t('Loading...')."',\n".
							   "\tloadingTitle : '".t('Click to cancel')."',\n".
							   "\tfocusTitle : '".t('Click to bring to front')."',\n".
							   "\tfullExpandTitle : '".t('Expand to actual size (f)')."',\n".
							   "\tpreviousText : '".t('Previous')."',\n".
							   "\tnextText : '".t('Next')."',\n".
							   "\tmoveText : '".t('Move')."',\n".
							   "\tcloseText : '".t('Close')."',\n".
							   "\tcloseTitle : '".t('Close (esc)')."',\n".
							   "\tresizeTitle : '".t('Resize')."',\n".
							   "\tplayText : '".t('Play')."',\n".
							   "\tplayTitle : '".t('Play slideshow (spacebar)')."',\n".
							   "\tpauseText : '".t('Pause')."',\n".
							   "\tpauseTitle : '".t('Pause slideshow (spacebar)')."',\n".
							   "\tpreviousTitle : '".t('Previous (arrow left)')."',\n".
							   "\tnextTitle : '".t('Next (arrow right)')."',\n".
							   "\tmoveTitle : '".t('Move')."',\n".
							   "\tfullExpandText : '".t('1:1')."',\n".
							   "\tnumber: '".t('Image')." %1 ".t('of')." %2',\n".
							   "\trestoreTitle : '".t('Click to close image, click and drag to move. Use arrow keys for next and previous.')."'";						    
					if ($this->credits) 
					{
						$script .= ",\n".
								   "\tcreditsText : '".$this->credits_title."',\n".
								   "\tcreditsTitle : '".$this->credits_text."'";
					}
					$script .= "\n};\n".
							   "hs.showCredits = ".$this->credits.";\n". 
							   "hs.graphicsDir = '".DIR_REL."/packages/highslide_web_albums/images/';\n". 
							   "hs.transitions = ['expand', 'crossfade'];\n". 
							   "hs.outlineType = 'rounded-white';\n". 
							   "hs.fadeInOut = true;\n".
							   "hs.align = 'center';\n";
					
					if ($this->dimming) 
						$script .= "hs.dimmingOpacity = 0.".$this->dimmingopacity.";\n";
						
					$script .= "hs.addSlideshow({\n".
							   "\tinterval: 5000,\n".
							   "\trepeat: false,\n".
							   "\tuseControls: true,\n".
							   "\tfixedControls: 'fit',\n".
							   "\toverlayOptions: {\n".
							   "\t\topacity: .75,\n".
							   "\t\tposition: 'bottom center',\n".
							   "\t\thideOnMouseOut: true\n".
							   "\t}\n".
							   "});\n";
				}
				
				$script .= "</script>\n";				
				$this->addFooterItem($script);
				
				if (strstr($this->getCurrentTemplate(), 'photowall'))
				{
					$this->addFooterItem($html->javascript('masonry.js', 'picasa_google_albums'));
					$this->addFooterItem($html->javascript('imagesloaded.js', 'picasa_google_albums'));
					$script = "<script>
								$(function(){
									$('#highslide_album_".$this->bID."').imagesLoaded( function(){
										$('#highslide_album_".$this->bID."').masonry({
											itemSelector : '.box'
										}).animate({opacity: 1}, 'slow');
									});
							    });
							   </script>";
					$this->addFooterItem($script);		   
				}
			}
		}
		
		function randomizeImages()
		{
			if($this->playback == 'RANDOM')
			{
				shuffle($this->images);
			}
			else if($this->playback == 'RANDOM-SET')
			{
				$imageGroups=array();
				$imageGroupIds=array();
				$sortedImgs=array();
				foreach($this->images as $imgInfo){
					$imageGroups[$imgInfo['groupSet']][]=$imgInfo;
					if( !in_array($imgInfo['groupSet'],$imageGroupIds) )
						$imageGroupIds[]=$imgInfo['groupSet'];
				}
				shuffle($imageGroupIds);
				foreach($imageGroupIds as $imageGroupId){
					foreach($imageGroups[$imageGroupId] as $imgInfo)
						$sortedImgs[]=$imgInfo;
				}
				$this->images = $sortedImgs;
			}
		}
		
		function recursiveArraySearch($needle, $haystack, $index = null) 
		{ 
			$aIt = new RecursiveArrayIterator($haystack); 
			$it = new RecursiveIteratorIterator($aIt); 
			
			while($it->valid()) 
			{        
				if (((isset($index) AND ($it->key() == $index)) OR (!isset($index))) AND ($it->current() == $needle)) { 
					return $aIt->key(); 
				} 
				
				$it->next(); 
			} 
			
			return false; 
		} 
		
		function prepareImage($img) 
		{			
			$fv = Loader::model('file_version');
			$ih = Loader::helper('image');

			$f = $t = File::getByID($img['fID']);
			$fp = $tp = new Permissions($f);
			
			$fv = $tv = $f->getApprovedVersion();	
				
			$image_size = $thumb_size = @getimagesize($f->getPath());
			
			$image['fID'] = $img['fID'];
			$image['groupSet'] = $img['groupSet'];
			$image['position'] = $img['position'];
						
			if (intval($this->thumb_attribute) == 1)
			{
				$attrib = FileAttributeKey::getByHandle('thumbnail');
				if (is_object($attrib)) 
				{	
					$vo = $f->getAttributeValueObject($attrib);		
					if (is_object($vo)) 
					{
						$file = $vo->getValue($attrib->akHandle);
						$t = File::getByID($file->fID);
						$tp = new Permissions($t);
						
						$tv = $t->getApprovedVersion();	
							
						$thumb_size = @getimagesize($t->getPath());
					}
				}
			}	
			
			$mw = $thumb_size[0];
			$mh = $thumb_size[1];							
			if ($thumb_size[0] > $this->thumb_width) {		
				$mw = $this->thumb_width > 0 ? $this->thumb_width : $thumb_size[0];
				
				if ($this->thumb_width_crop != 0)
					$mh = $this->thumb_width;
				else
					$mh = round(($thumb_size[1] / $thumb_size[0]) * $mw);
			}
			$image['thumb'] = $ih->getThumbnail($t, $mw, $mh, $this->thumb_width_crop);				
			
			$mw = $thumb_size[0];
			$mh = $thumb_size[1];	
			if ($image_size[0] > $this->max_width) {		
				$mw = $this->max_width > 0 ? $this->max_width : $image_size[0];
				
				if ($this->max_width_crop != 0)
					$mh = $max_width;
				else
					$mh = round(($image_size[1] / $image_size[0]) * $mw);
			}			
			$image['full'] = $ih->getThumbnail($f, $mw, $mh, $this->max_width_crop);
					
			if ($this->alttags && sizeof($this->selected_attributes) > 0) 
			{
				$image['class'] = '_large';
				if ($this->alttags_title) 
					$image['class'] = '_small';
										
				foreach ($this->selected_attributes as $sa) {	
					$attrib = FileAttributeKey::getByHandle($sa);
					if (is_object($attrib)) {	
						$vo = $f->getAttributeValueObject($attrib);		
						if (is_object($vo)) {
							$image['caption'][] = array('title' => $attrib->akName,
														'value' => $vo->getValue($attrib->akHandle));
						}
					}
					elseif ($sa == 'fvFileID' && trim($fv->getFileID()) != '') 
						$image['caption'][] = array('title' => substr($sa, 2),
													'value' => $fv->getFileID());
					elseif ($sa == 'fvFilename' && trim($fv->getFileName()) != '') 
						$image['caption'][] = array('title' => substr($sa, 2),
													'value' => $fv->getFileName());	
					elseif ($sa == 'fvSize' && trim($fv->getSize()) != '') 
						$image['caption'][] = array('title' => substr($sa, 2),
													'value' => $fv->getSize());									
					elseif ($sa == 'fvDateAdded' && trim($fv->getDateAdded()) != '') 
						$image['caption'][] = array('title' => 'Date Added',
													'value' => $fv->getDateAdded());
					elseif ($sa == 'fvTitle' && trim($fv->getTitle()) != '') 
						$image['caption'][] = array('title' => substr($sa, 2),
													'value' => $fv->getTitle());
					elseif ($sa == 'fvDescription' && trim($fv->getDescription()) != '') 
						$image['caption'][] = array('title' => substr($sa, 2),
													'value' => $fv->getDescription());
					elseif ($sa == 'fvTags' && trim($fv->getTags()) != '') 
						$image['caption'][] = array('title' => substr($sa, 2),
													'value' => $fv->getTags());
					elseif ($sa == 'fvAuthor' && trim($fv->getAuthorName()) != '') 
						$image['caption'][] = array('title' => substr($sa, 2),
													'value' => $fv->getAuthorName());
					elseif ($sa == 'fvDownload') 
						$image['caption'][] = array('title' => substr($sa, 2),
													'value' => '<a href="'.$fv->getDownloadURL().'">Download File</a>');
				}
			}
			return $image;		
		}
		
		function getCurrentTemplate()
		{
			$blockObject = $this->getBlockObject();
			if (is_object($blockObject)) {
				return $blockObject->getBlockFilename();
			}
			return '';
		}
	}

?>
